bl_info = {
    "name": "Copy Object Weight",
    "author": "Mathias Aubry",
    "version": (1, 0),
    "blender": (2, 6, 4),
    "location": "View3D > Tool Shelf > Copy Object Weight",
    "description": "Find the weight for each of an object from another - Active object is the destination one",
    "category": "Rigging"}

import bpy, math
import datetime
from bpy.props import *
from bpy.types import Operator, Panel

def copy_weight_obj(tol,delgrp):
    print(delgrp)
    print("start")
    tol = .1
    step = 0.001
    tol_ori = tol
    start = (datetime.datetime.now())
    
    
    if bpy.context.object.mode == 'OBJECT' and len(bpy.context.selected_objects) == 2:
        obj_dest = bpy.context.active_object
        obj_ori = bpy.context.selected_objects[1]
    
        if obj_dest == obj_ori:
            obj_ori = bpy.context.selected_objects[0]
    
        msh_dest = obj_dest.data
        msh_ori = obj_ori.data
        
        ###delete old groups if check
        
        if delgrp==True:
            for grp in obj_dest.vertex_groups:
                obj_dest.vertex_groups.remove(grp)
        
        
        od_wm = obj_dest.matrix_world
        oo_wm = obj_ori.matrix_world    
        
        ##let's now add the groups!
        DictGroupOri = {} ##ask the name of the group it will give you the index
        i=0
        for group in obj_ori.vertex_groups:
            DictGroupOri[obj_ori.vertex_groups[i].name]=obj_ori.vertex_groups[i]
            i+=1
            
        DictGroupDest = {} ##ask the name of the group it will give you the index
        i=0
        for group in obj_dest.vertex_groups:
            DictGroupDest[group.name]=i
            i+=1    
        
        i=0        
        for g in DictGroupOri:
            if not g in DictGroupDest:
                obj_dest.vertex_groups.new(obj_ori.vertex_groups[i].name) ## and here we make sure they are added in the same order!
            i+=1        

        
        DictGroupDestINV = {} ##ask the name of the group it will give you the index 
           
        i=0
        for group in obj_dest.vertex_groups:
            DictGroupDestINV[group.index]=group
            i+=1
        print(len(DictGroupDestINV))
        print(len(DictGroupOri))
        
        for vd in bpy.data.meshes[msh_dest.name].vertices:
            
            #print(vd.index)
            total=0
            EachVertexOri = {}
            i=0
            while (len(EachVertexOri) == 0):
                for vo in bpy.data.meshes[obj_ori.data.name].vertices:
                    
                    ## find the world coordinate
                    gco_vo = oo_wm*vo.co
                    gco_vd = od_wm*vd.co
                    
                    
                    if gco_vo[2]-tol<gco_vd[2]<gco_vo[2]+tol:
                        if gco_vo[1]-tol<gco_vd[1]<gco_vo[1]+tol:
                            if gco_vo[0]-tol<gco_vd[0]<gco_vo[0]+tol:
                                
                                distance = math.sqrt(math.pow((gco_vd[0]-gco_vo[0]),2)+math.pow((gco_vd[1]-gco_vo[1]),2)+math.pow((gco_vd[2]-gco_vo[2]),2)) 
                                #print("vo",vo.index)
                                        
                                if distance<tol:
                                ### create a dictionnary on vertex and there distance
                                    EachVertexOri[vo.index]=distance
                                    total+=math.pow(distance,2) ### distance total  (pow is to favorise those closer)
                                        
                
                              
                tol+=(step*i)
                i+=1
                
            tol=tol_ori                            
                                                
            for entry in EachVertexOri:
                factor=1
                
                if total>0 and len(EachVertexOri)>1:
                    dist=EachVertexOri[entry]           
                    factor_step1 =(math.pow(dist,2)*(1/total))
                    factor_step2 =1/len(EachVertexOri)-factor_step1
                    factor =1/len(EachVertexOri)+factor_step2        
                
                for groups in msh_ori.vertices[entry].groups:
                    grp_name=obj_ori.vertex_groups[groups.group].name
                    grp_weight=groups.weight*factor               
                    obj_dest.vertex_groups[grp_name].add([vd.index],grp_weight,'ADD')
    				
    end = (datetime.datetime.now())
    timing = end-start
    print("end =",timing)

def copy_weight_object_start():
    if len(bpy.context.selected_objects)==2:
        window_copy_weight_object()
    else:
        window_error_mesh()



def window_copy_weight_object():
    class DialogOperator(bpy.types.Operator):
        bl_idname = "object.dialog_operator"
        bl_label = "Copy vertex Weight - Active object is destination!"
                
        tolerance = FloatProperty(name="Tolerance", min=0, max=100, precision=3, default=0.001)
        delete_grp = BoolProperty(name="Delete previous group?") 

     
        def execute(self, context):
            copy_weight_obj(self.tolerance,self.delete_grp)
            return {'FINISHED'}
     
        def invoke(self, context, event):
            return context.window_manager.invoke_props_dialog(self)
     
     
    bpy.utils.register_class(DialogOperator)
     
    # Invoke the dialog when loading
    bpy.ops.object.dialog_operator('INVOKE_DEFAULT')

def window_error_mesh(): 
    class DialogOperator(bpy.types.Operator):
        bl_idname = "object.dialog_operator"
        bl_label = "Error - Need to have 2 object selected in Object mode"

     
        def execute(self, context):
            return {'FINISHED'}
     
        def invoke(self, context, event):
            return context.window_manager.invoke_props_dialog(self)
     
     
    bpy.utils.register_class(DialogOperator)
     
    # Invoke the dialog when loading
    bpy.ops.object.dialog_operator('INVOKE_DEFAULT')

### the panel in the tool view port    
class VIEW3D_CopyObjectWeight(Panel):
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'TOOLS'
    bl_label = "Copy Object Weight"
    bl_context = "objectmode"
    bl_options = {'DEFAULT_CLOSED'}

    def draw(self, context):
        active_obj = context.active_object
        layout = self.layout
        col = layout.column(align=True)
        col.operator("my.copyweightobj")

## the button itself        
class OBJECT_OT_Copy_weight_obj(bpy.types.Operator):
    bl_idname = "my.copyweightobj"
    bl_label = "Copy Weight Object"
    def execute(self, context):
        copy_weight_object_start()
        return{'FINISHED'}   

def register():
    bpy.utils.register_module(__name__)
    
if __name__ == "__main__":
    register()
    register()